@@ -1,15 +1,13 @@
-#src/resources/Royalty.py
+#src/story_protcol_python_sdk/resources/Royalty.py
 
-import logging
 from web3 import Web3
+
 from story_protocol_python_sdk.abi.IPAssetRegistry.IPAssetRegistry_client import IPAssetRegistryClient
 from story_protocol_python_sdk.abi.IpRoyaltyVaultImpl.IpRoyaltyVaultImpl_client import IpRoyaltyVaultImplClient
 from story_protocol_python_sdk.abi.RoyaltyPolicyLAP.RoyaltyPolicyLAP_client import RoyaltyPolicyLAPClient
 from story_protocol_python_sdk.abi.RoyaltyModule.RoyaltyModule_client import RoyaltyModuleClient
 
-# Configure logging
-# logging.basicConfig(level=logging.INFO)
-# logger = logging.getLogger(__name__)
+from story_protocol_python_sdk.utils.transaction_utils import build_and_send_transaction
 
 class Royalty:
     def __init__(self, web3: Web3, account, chain_id):
@@ -29,43 +27,28 @@ class Royalty:
                 raise ValueError(f"The parent IP with id {parent_ip_id} is not registered.")
 
             # Get the royalty vault address
-            # print("gonna check for royalty vault addy")
             proxy_address = self._getRoyaltyVaultAddress(child_ip_id)
-            # print("The proxy address: ", proxy_address)
 
             # Initialize the IP Royalty Vault client with the proxy address
             ip_royalty_vault_client = IpRoyaltyVaultImplClient(self.web3, contract_address=proxy_address)
 
-            # Build the transaction
-            transaction = ip_royalty_vault_client.build_collectRoyaltyTokens_transaction(parent_ip_id, {
-                'from': self.account.address,
-                'nonce': self.web3.eth.get_transaction_count(self.account.address),
-                'gas': 2000000,
-                'gasPrice': self.web3.to_wei('300', 'gwei')
-            })
-
-            # Sign the transaction using the account object
-            signed_txn = self.account.sign_transaction(transaction)
-            # logger.info(f"Signed transaction: {signed_txn}")
-
-            # Send the transaction
-            tx_hash = self.web3.eth.send_raw_transaction(signed_txn.rawTransaction)
-            # logger.info(f"Transaction hash: {tx_hash.hex()}")
-
-            # Wait for transaction receipt with a longer timeout
-            tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash, timeout=600)  # 10 minutes timeout
-            # logger.info(f"Transaction receipt: {tx_receipt}")
+            # Build and send the transaction
+            response = build_and_send_transaction(
+                self.web3,
+                self.account,
+                ip_royalty_vault_client.build_collectRoyaltyTokens_transaction,
+                parent_ip_id,
+                tx_options=tx_options
+            )
     
-            royaltyTokensCollected = self._parseTxRoyaltyTokensCollectedEvent(tx_receipt)
-            # print("Number of royalty tokens collected: ", royaltyTokensCollected)
+            royaltyTokensCollected = self._parseTxRoyaltyTokensCollectedEvent(response['txReceipt'])
 
             return {
-                'txHash': tx_hash.hex(),
+                'txHash': response['txHash'],
                 'royaltyTokensCollected': royaltyTokensCollected
             }
         
         except Exception as e:
-            # logger.error(f"Failed to collect royalty tokens: {e}")
             raise e
 
     def _getRoyaltyVaultAddress(self, royalty_vault_ip_id):
@@ -99,41 +82,25 @@ class Royalty:
         try:
             # Get the royalty vault address
             proxy_address = self._getRoyaltyVaultAddress(child_ip_id)
-            # logger.info(f"The proxy address: {proxy_address}")
-            #print("Thne proxy addres: ", proxy_address)
 
             # Initialize the IP Royalty Vault client with the proxy address
             ip_royalty_vault_client = IpRoyaltyVaultImplClient(self.web3, contract_address=proxy_address)
 
-            # Build the transaction
-            transaction = ip_royalty_vault_client.build_snapshot_transaction({
-                'from': self.account.address,
-                'nonce': self.web3.eth.get_transaction_count(self.account.address),
-                'gas': 2000000,
-                'gasPrice': self.web3.to_wei('300', 'gwei')
-            })
+            # Build and send the transaction
+            response = build_and_send_transaction(
+                self.web3,
+                self.account,
+                ip_royalty_vault_client.build_snapshot_transaction,
+                tx_options=tx_options
+            )
 
-            # Sign the transaction using the account object
-            signed_txn = self.account.sign_transaction(transaction)
-            # logger.info(f"Signed transaction: {signed_txn}")
-
-            # Send the transaction
-            tx_hash = self.web3.eth.send_raw_transaction(signed_txn.rawTransaction)
-            # logger.info(f"Transaction hash: {tx_hash.hex()}")
-
-
-            # Wait for transaction receipt with a longer timeout
-            tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash, timeout=600)  # 10 minutes timeout
-            # logger.info(f"Transaction receipt: {tx_receipt}")
-
-            snapshotId =  self._parseTxSnapshotCompletedEvent(tx_receipt)
+            snapshotId =  self._parseTxSnapshotCompletedEvent(response['txReceipt'])
 
             return {
-                'txHash': tx_hash.hex(),
+                'txHash': response['txHash'],
                 'snapshotId': snapshotId
             }
         except Exception as e:
-            # logger.error(f"Failed to snapshot: {e}")
             raise e
 
     def _parseTxSnapshotCompletedEvent(self, tx_receipt):
@@ -150,12 +117,10 @@ class Royalty:
 
         return None
 
-    def claimableRevenue(self, child_ip_id, account_address, snapshot_id, token, tx_options=None):
+    def claimableRevenue(self, child_ip_id, account_address, snapshot_id, token):
         try:
             # Get the royalty vault address
             proxy_address = self._getRoyaltyVaultAddress(child_ip_id)
-            # logger.info(f"The proxy address: {proxy_address}")
-            # print("Thne proxy addres: ", proxy_address)
 
             # Initialize the IP Royalty Vault client with the proxy address
             ip_royalty_vault_client = IpRoyaltyVaultImplClient(self.web3, contract_address=proxy_address)
@@ -170,7 +135,6 @@ class Royalty:
             return claimable_revenue
 
         except Exception as e:
-            # logger.error(f"Failed to calculate claimable revenue: {e}")
             raise e
         
     def payRoyaltyOnBehalf(self, receiver_ip_id, payer_ip_id, token, amount, tx_options=None):
@@ -184,75 +148,50 @@ class Royalty:
             is_payer_registered = self.ip_asset_registry_client.isRegistered(payer_ip_id)
             if not is_payer_registered:
                 raise ValueError(f"The payer IP with id {payer_ip_id} is not registered.")
-            
-            # Build the transaction
-            transaction = self.royalty_module_client.build_payRoyaltyOnBehalf_transaction(receiver_ip_id, payer_ip_id, token, amount, {
-                'from': self.account.address,
-                'nonce': self.web3.eth.get_transaction_count(self.account.address),
-                'gas': 2000000,
-                'gasPrice': self.web3.to_wei('300', 'gwei')
-            })
-
-            # Sign the transaction using the account object
-            signed_txn = self.account.sign_transaction(transaction)
-            # logger.info(f"Signed transaction: {signed_txn}")
-
-            # Send the transaction
-            tx_hash = self.web3.eth.send_raw_transaction(signed_txn.rawTransaction)
-            # logger.info(f"Transaction hash: {tx_hash.hex()}")
-
-            # Wait for transaction receipt with a longer timeout
-            tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash, timeout=600)  # 10 minutes timeout
-            # logger.info(f"Transaction receipt: {tx_receipt}")
-
-            if tx_options and tx_options.get('wait_for_transaction'):
-                self.web3.eth.wait_for_transaction_receipt(tx_hash)
-
-            return {'txHash': tx_hash.hex()}
+                
+            # Build and send the transaction
+            response = build_and_send_transaction(
+                self.web3,
+                self.account,
+                self.royalty_module_client.build_payRoyaltyOnBehalf_transaction,
+                receiver_ip_id,
+                payer_ip_id,
+                token,
+                amount,
+                tx_options=tx_options
+            )
 
+            return {'txHash': response['txHash']}
+        
         except Exception as e:
-            # logger.error(f"Failed to pay royalty on behalf: {e}")
             raise e
     
-    def claimRevenue(self, snapshotIds, child_ip_id, token):
+    def claimRevenue(self, snapshot_ids, child_ip_id, token, tx_options=None):
         try:
             # Get the royalty vault address
             proxy_address = self._getRoyaltyVaultAddress(child_ip_id)
-            # logger.info(f"The proxy address: {proxy_address}")
-            # print("The proxy addres: ", proxy_address)
 
             # Initialize the IP Royalty Vault client with the proxy address
             ip_royalty_vault_client = IpRoyaltyVaultImplClient(self.web3, contract_address=proxy_address)
 
-            # Build the transaction
-            transaction = ip_royalty_vault_client.build_claimRevenueBySnapshotBatch_transaction(snapshotIds, token, {
-                'from': self.account.address,
-                'nonce': self.web3.eth.get_transaction_count(self.account.address),
-                'gas': 2000000,
-                'gasPrice': self.web3.to_wei('300', 'gwei')
-            })
+            # Build and send the transaction
+            response = build_and_send_transaction(
+                self.web3,
+                self.account,
+                ip_royalty_vault_client.build_claimRevenueBySnapshotBatch_transaction,
+                snapshot_ids,
+                token,
+                tx_options=tx_options
+            )
 
-            # Sign the transaction using the account object
-            signed_txn = self.account.sign_transaction(transaction)
-            # logger.info(f"Signed transaction: {signed_txn}")
-
-            # Send the transaction
-            tx_hash = self.web3.eth.send_raw_transaction(signed_txn.rawTransaction)
-            # logger.info(f"Transaction hash: {tx_hash.hex()}")
-
-            # Wait for transaction receipt with a longer timeout
-            tx_receipt = self.web3.eth.wait_for_transaction_receipt(tx_hash, timeout=600)  # 10 minutes timeout
-            # logger.info(f"Transaction receipt: {tx_receipt}")
-
-            revenue_tokens_claimed =  self._parseTxRevenueTokenClaimedEvent(tx_receipt)
-            # print("Amount: ", revenue_tokens_claimed)
+            revenue_tokens_claimed = self._parseTxRevenueTokenClaimedEvent(response['txReceipt'])
 
             return {
-                'txHash': tx_hash.hex(),
+                'txHash': response['txHash'],
                 'claimableToken': revenue_tokens_claimed
             }
+    
         except Exception as e:
-            # logger.error(f"Failed to claim revenue: {e}")
             raise e
         
     def _parseTxRevenueTokenClaimedEvent(self, tx_receipt):
@@ -261,7 +200,6 @@ class Royalty:
         for log in tx_receipt['logs']:
             if log['topics'][0].hex() == event_signature:
                 data = log['data']
-                # print("the data is ", data)
 
                 # Convert the last 32 bytes to an integer
                 revenue_tokens_claimed = int.from_bytes(data[-32:], byteorder='big')
